﻿using CallCenter.Caches;
using CallCenter.Calls;
using CallCenter.Realtimes;
using CallCenter.Users;
using Microsoft.AspNetCore.SignalR;
using XF.Domain.Dependency;
using XF.Domain.Exceptions;

namespace CallCenter.Api.Realtimes
{
    public class RealtimeService : IRealtimeService, IScopeDependency
    {
        private readonly IHubContext<CallCenterHub> _hubContext;
        private readonly IUserCacheManager _userCacheManager;


        public RealtimeService(IHubContext<CallCenterHub> hubContext, IUserCacheManager userCacheManager)
        {
            _hubContext = hubContext;
            _userCacheManager = userCacheManager;
        }

        /// <summary>
        /// 振铃事件通知
        /// </summary>
        /// <param name="userId"></param>
        /// <param name="dto"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        /// <exception cref="NotImplementedException"></exception>
        public async Task RingAsync(string userId, RingDto dto, CancellationToken cancellationToken)
        {
            var work = _userCacheManager.GetWorkByUser(userId);
            if (string.IsNullOrEmpty(work.SignalRId))
                throw new UserFriendlyException($"无效signalr.connectionId, workId:{work.Id}, userId: {userId}");
            await _hubContext.Clients.Client(work.SignalRId).SendAsync(RealtimeMethods.Ring, dto, cancellationToken);
        }

        /// <summary>
        /// 应答事件通知
        /// </summary>
        /// <param name="userId"></param>
        /// <param name="dto"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        /// <exception cref="NotImplementedException"></exception>
        public async Task AnsweredAsync(string userId, AnseredDto dto, CancellationToken cancellationToken)
        {
            var work = _userCacheManager.GetWorkByUser(userId);
            if (string.IsNullOrEmpty(work.SignalRId))
                throw new UserFriendlyException("无效signalr.connectionId");
            await _hubContext.Clients.Client(work.SignalRId).SendAsync(RealtimeMethods.Answered, dto, cancellationToken);
        }

        /// <summary>
        /// 电话挂断事件通知
        /// </summary>
        /// <param name="userId"></param>
        /// <param name="dto"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        /// <exception cref="NotImplementedException"></exception>
        public async Task ByeAsync(string userId, ByeDto dto, CancellationToken cancellationToken)
        {
            var work = _userCacheManager.GetWorkByUser(userId);
            if (string.IsNullOrEmpty(work.SignalRId))
                throw new UserFriendlyException("无效signalr.connectionId");
            await _hubContext.Clients.Client(work.SignalRId).SendAsync(RealtimeMethods.Bye, dto, cancellationToken);
        }

        /// <summary>
        /// 通话队列通知
        /// </summary>
        /// <param name="userId"></param>
        /// <param name="count"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public async Task CallQueueAsync(List<Call> list, CancellationToken cancellationToken)
        {
            var works = _userCacheManager.GetWorks();
            if (works != null)
            {
                var sendlist = works.Where(x => x.SignalRId != null && x.SignalRId != "").Select(x => x.SignalRId).ToList();
                foreach (var item in sendlist)
                {
                    await _hubContext.Clients.Client(item).SendAsync(RealtimeMethods.CallQueue, new { list }, cancellationToken);
                }
            }
        }

        /// <summary>
        /// 回铃通知（去电）
        /// </summary>
        /// <param name="userId"></param>
        /// <param name="dto"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        /// <exception cref="UserFriendlyException"></exception>
        public async Task AlertAsync(string userId, AlertDto dto, CancellationToken cancellationToken)
        {
            var work = _userCacheManager.GetWorkByUser(userId);
            if (string.IsNullOrEmpty(work.SignalRId))
                throw new UserFriendlyException("无效signalr.connectionId");
            await _hubContext.Clients.Client(work.SignalRId).SendAsync(RealtimeMethods.Alert, dto, cancellationToken);
        }


        /// <summary>
        /// 话机空闲通知
        /// </summary>
        /// <param name="userId"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public async Task IdleAsync(string userId, CancellationToken cancellationToken)
        {
            var work = _userCacheManager.GetWorkByUser(userId);
            if (string.IsNullOrEmpty(work.SignalRId))
                throw new UserFriendlyException("无效signalr.connectionId");
            await _hubContext.Clients.Client(work.SignalRId).SendAsync(RealtimeMethods.Idle, new { EventName = "Idle" }, cancellationToken);
        }
    }
}
